package de.gruschke.dasync;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

// import gnu.getopt.LongOpt;
import gnu.getopt.Getopt;

/**
 * Main program of DASync
 *
 * @author Boris Gruschke
*/

public class Main {

/**
 * Usage: dasync [-v] [-c PrintDiff|UpdateDS|UpdateAS] <it>DataSetFactory</it>.
 * Invokes {@link #runComparison runComparison(...)}.
 */
    public static void main(String[] args)
    throws ClassNotFoundException,
	InstantiationException,
	IllegalAccessException,
	IOException,
	NamingException,
	ConfigurationException,
	Exception /* thrown by getFromAS/DS */ {
	// three modes: printDiff, updateDS and updateAS
	// switches: batch vs. interactive, (verbose)

	//
	// Command Line Processing
	//
	//	LongOpt[] longopt = new LongOpt[1];

	//	longopts[0] = new LongOpt("help",LongOpt.NO_ARGUMENT, null,'h');

	Getopt g = new Getopt("dasync", args,"hvc:");  // , longopts);

	boolean verbose = false;
	String command = "PrintDiff";
	final String usage = "dasync [-v] [-c <command>] <dataSetFactory> where command is PrintDiff, UpdateDS or UpdateAS";

	int c;
	while (( c = g.getopt()) != -1 ) {
	    switch(c) {
	    case 'h':
		System.out.println("usage: "+ usage);
		System.exit(1);
		break;
	    case 'v':
		verbose = true;
		break;
	    case 'c':
		command = g.getOptarg();
		break;
	    default:
		System.out.println("Invalid option");
		break;
	    }
	}

	// Setting up the dataSetFactory as specified in the command line
	//
	int dsfClassNameArgIndex = g.getOptind();
	if (dsfClassNameArgIndex == args.length) {
	    System.out.println("usage: " + usage);
	    System.exit(2);
	}
	String dataSetFactoryClassName = args[dsfClassNameArgIndex];
	if (verbose) {
	    System.out.println("using dataSetFactory " +
			       dataSetFactoryClassName + ".");
	}

	Class dataSetFactoryClass = Class.forName(dataSetFactoryClassName);
	DataSetFactory dsf = null;
	try {
	    dsf = (DataSetFactory)dataSetFactoryClass.newInstance();
	} catch(ClassCastException oops) {
	    System.err.println("Programming Error: DataSetFactory "
			       + dataSetFactoryClassName
			       + " has to implement interface de.gruschke.dasync.DataSetFactory.");
	    System.exit(-1);
	}

	// Reading Arguments for getFromAS/DS-Methods from the command line
	//
	Map dsfArgs = new HashMap();
	for (int i=g.getOptind()+1;i<args.length;i++) {
	    int sep = args[i].indexOf('=');
	    if (sep == -1) {
		System.err.println("Skipping wrong argument" + args[i] + "!");
		continue;
	    }
	    dsfArgs.put(args[i].substring(0,sep),args[i].substring(sep+1));
	}

	// Setting up the mapComparer as specified by the command option
	//
	Class mapComparerClass = Class.forName("de.gruschke.dasync." + command + "MapComparer");
	MapComparer mc = (MapComparer)mapComparerClass.newInstance();

	runComparison(mc, dsf, dsfArgs, verbose, command);
    }

    /**
      *  Entry point for running DASync from within Java Programs. It does the
      * initialization and runs {@link de.gruschke.dasync.MapComparer#compare MapComparer.compare(...)}.
      *
      * @param mc Instance of a subclass of {@link de.gruschke.dasync.MapComparer MapComparer}.
      * 		The MapComparer determines "what to do".
      * @param dsf Instance of a (application provided) class implementing DataSetFactory.
      * 		{@link de.gruschke.dasync.DataSetFactory#init DataSetFactory.init(...)} is
      *		invoked from runComparison(...).
      * @param dsfArgs Arguments for the DataSetFactory
      * @param verbose Screw up debugging output
      * @param command One of "PrintDiff", "UpdateDS", "UpdateAS". Actually a prefix of
      * 		MapComparer class name.
      */
    public static void runComparison(MapComparer mc,
				     DataSetFactory dsf,
				     Map dsfArgs,
				     boolean verbose,
				     String command)
    throws ConfigurationException,
	IOException,
	NamingException,
	Exception /* thrown by getFromAS/DS */ {

	if (verbose) {
	    mc.setVerbose();

	    System.out.println("Getting initial JNDI context of the application server");
	}

	String asPropFileName = System.getProperty("dasync.as.propfile");
	if (asPropFileName == null) {
	    throw new ConfigurationException("Property dasync.as.propfile must be set to the filename of a property file that configures the application server's JNDI environment.");
	}

	Properties asProps = new Properties();

        try {
	    asProps.load(new FileInputStream(asPropFileName));
        }
        catch (IOException oops) {
          throw new ConfigurationException("FAILED to load Property file "
 					   + asPropFileName + ".");
        }
	Context initialAppServerContext = new InitialContext(asProps);

	if (verbose) {
	    System.out.println("Getting initial JNDI context of the directory server");
	}

	String dsPropFileName = System.getProperty("dasync.ds.propfile");
	if (dsPropFileName == null) {
	    throw new ConfigurationException("Property dasync.ds.propfile must be set to the filename of a property file that configures the directory server's JNDI environment.");
	}

	Properties dsProps = new Properties();
        try {
	    dsProps.load(new FileInputStream(dsPropFileName));
        }
        catch (IOException oops) {
          throw new ConfigurationException("FAILED to load Property file "
 					   + dsPropFileName + ".");
        }
	DirContext initialDirServerContext = null;
	try {
	    initialDirServerContext = new InitialDirContext(dsProps);
	}
	catch (Exception e) {
	    System.err.println("ERROR: Connecting to the Directory Server FAILED");
	    dsProps.list(System.err);
	    throw e;
	}

	// Initialize DataSetFactory
	//
	if (verbose) {
	    System.out.println("Initializing DataSetFactory.");
	}
	dsf.init(initialAppServerContext,
		 initialDirServerContext,
		 dsfArgs,
		 verbose,
		 command);

	// Read data
	//
	if (verbose) {
	    System.out.println("Reading data from the application server.");
	}

	Map mapFromAS = dsf.getFromAS();

	if (verbose) {
	    if (mapFromAS == null) {
		System.out.println("Got no data rows.");
	    }
	    else {
		System.out.println("Got " + mapFromAS.size() + " data rows.");
	    }
	    System.out.println("Reading data from the directory server.");
	}

	Map mapFromDS = dsf.getFromDS();

	if (verbose) {
	    if (mapFromDS == null) {
		System.out.println("Got no data rows.");
	    }
	    else {
		System.out.println("Got " + mapFromDS.size() + " data rows.");
	    }
	}

	// Run the Comparer
	//
	if (verbose) {
	    System.out.println("Computing and executing differences.");
	}
	mc.compare(mapFromAS, mapFromDS);
	if (verbose) {
	    System.out.println("Done.");
	}
    }

}
