package de.gruschke.dasync;

import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Functionality common to all sorts of synchronization jobs.
 *
 * @author Boris Gruschke
 */
public abstract class MapComparer {
/**
 * Computes comparison of two maps
 *
 * Compare invokes the methods {@link #inOneOnly(Object, Object) inOneOnly(...)},
 * {@link #inTwoOnly(Object, Object) inTwoOnly(...)} if a key appears in one of the
 * maps only, {@link #oneAndTwoDiffer(Object, Object, Object) oneAndTwoDiffer(...)} if
 * a key appears in both maps but with different values and
 * {@link #oneAndTwoEqual(Object, Object) oneAndTwoEqual(...)} if a key appears in both
 * maps and both values are equal.
 * @param one Map of Keys and {@link de.gruschke.dasync.DataRow} objects
 * @param two Map of Keys and {@link de.gruschke.dasync.DataRow} objects
 */
    public void compare(Map one, Map two) {
	if (one == null) { one = new HashMap(); }
	if (two == null) { two = new HashMap(); }
	Set keysOne = one.keySet();
	Set keysTwo = two.keySet();

	// Maps Differ and Map One Only
	//
	Iterator itOverOne = keysOne.iterator();
	while (itOverOne.hasNext()) {
	    Object key = itOverOne.next();
	    Object valueOne = one.get(key);

	    if (keysTwo.contains(key)) {
		Object valueTwo = two.get(key);
		if (valueOne.equals(valueTwo)) {
		    oneAndTwoEqual(key, valueOne);
		}
		else {
		    oneAndTwoDiffer(key, valueOne, valueTwo);
		}
	    }
	    else {
		inOneOnly(key,valueOne);
	    }
	}

	// Map Two Only
	//
	Iterator itOverTwo = keysTwo.iterator();
	while (itOverTwo.hasNext()) {
	    Object key = itOverTwo.next();
	    if (!keysOne.contains(key)) {
		inTwoOnly(key,two.get(key));
	    }
	}
    }

    /**
      * screw up the debugging
      */
    public void setVerbose() {
	this.verbose = true;
    }

    protected boolean verbose = false;

  /**
   * Key appears in Map one only
   * @param key key object from Map one
   * @param value {@link de.gruschke.dasync.DataRow} object from Map one
   */
    protected abstract void inOneOnly(Object key, Object value);

  /**
   * Key appears in Map two only
   * @param key key object from Map two
   * @param value {@link de.gruschke.dasync.DataRow} object from Map two
   */
    protected abstract void inTwoOnly(Object key, Object value);

  /**
   * Key appears in both Maps but with different values
   * @param key key object from one of the Maps
   * @param valueOne {@link de.gruschke.dasync.DataRow} object from Map one
   * @param valueTwo {@link de.gruschke.dasync.DataRow} object from Map two
   */
    protected abstract void oneAndTwoDiffer(Object key, Object valueOne, Object valueTwo);

  /**
   * Key appears in both Maps with equal values
   * @param key key object from one of the Maps
   * @param value {@link de.gruschke.dasync.DataRow} object from one of the Maps
   */
    protected abstract void oneAndTwoEqual(Object key, Object value);
}
